<template>
  <MenuReaction v-if="isShowReaction && showEmoji && status !== 'failed'" :id="id" :is-seller="isSeller" @onSuccess="handleReactionSuccess" @onActive="handleOnActive"/>
  <ChatRepliedMessage v-if="reply_from_id && object?.replied_message && status !== 'loading'" :replied-message="object?.replied_message" class="cursor-pointer" @click="viewReply()"/>
  <ReactionIcon :reaction-list="reactionArr"/>
  <span v-html="message" class="overflow-hidden relative block break-words"></span>
  <FailedSendMessage v-if="isSeller && status ==='failed'" :message="wa_error_message"/>

  <MenuChatBubble v-if="isShowMenuChat" :is-seller="isSeller" :message="message" :id="id" :chat_type="chatType"/>

  <div v-if="is_forwarded === 1"
        class="absolute  -top-8 gap-1 rounded-tl-xl rounded-tr-xl left-0 block w-full  ">
    <div class="relative flex py-2 items-center">
      <svg xmlns="http://www.w3.org/2000/svg" class="w-5 h-5 fill-current text-gray-400"
            viewBox="0 0 24 24">
        <path d="M21,12L14,5V9C7,10 4,15 3,20C5.5,16.5 9,14.9 14,14.9V19L21,12Z"/>
      </svg>
      <span class="italic text-gray-400">Forwarded</span>
    </div>
  </div>
  <div :class="`relative flex items-end gap-5 ${assignee ? 'justify-between' : 'justify-end'}` ">
    <div v-if="isFromApplication && isSeller" class="flex items-center gap-2 whitespace-nowrap  rounded-full py-1 px-2 bottom-1 text-xs bg-white text-primary text-right select-none">
      <span class="w-2 h-2 bg-scheme-primary rounded-full"></span>
      <span>{{ applicationAgentName }}</span>
    </div>
    <div v-else-if="assignee"
          class="flex items-center gap-2 whitespace-nowrap  rounded-full py-1 px-2 bottom-1 text-xs bg-white text-primary text-right select-none">
      <span class="w-2 h-2 bg-scheme-primary rounded-full"></span>
      <span>{{ assignee.full_name }}</span>
    </div>
    <div class="flex gap-1 items-center justify-end">
      <span class="block  text-xs text-gray-600 text-right select-none">{{ date }}</span>
      <!-- loading -->
      <template v-if="isSeller && !isLoader">
        <IcMessageStatus :status="status"/>
      </template>
    </div>
  </div>
  <div class="relative block pt-2 w-full border-t border-scheme-primary mt-2 border-opacity-10 text-center">
    <button class="btn !h-8 min-h-0 btn-link !no-underline rounded-0 normal-case relative block w-full font-medium text-sm text-scheme-primary" v-if="buttonLabel">{{buttonLabel}}</button>
  </div>
</template>

<script setup lang="ts">
import FailedSendMessage from "../Utility/FailedSendMessage.vue";
import IcPersonGeneral from "../Icon/IcPersonGeneral.vue";
import IcMessageStatus from "../Icon/IcMessageStatus.vue";
import {onMounted, ref, watch} from "vue";
import MenuChatBubble from "../Menu/MenuChatBubble.vue";
import helpers from "../../utils/helpers";
import ChatRepliedMessage from "./ChatRepliedMessage.vue";
import ReactionIcon from "../Icon/ReactionIcon.vue";
import MenuReaction from "../../components/Menu/MenuReaction.vue";
import {useChatListStore} from "../../stores/chat_list";
import {useGeneralStore} from "../../stores/general";
import {useArrayFind, useArrayFindIndex} from "@vueuse/core";

interface Props {
  isSeller?: boolean
  isError?: boolean
  isLoading?: boolean
  isShowReaction?: boolean
  isShowMenuChat?: boolean
  status?: string
  isLoader?: boolean
  id?: bigint
  date?: string
  is_forwarded?: number
  reply_from_id?: string | number
  message?: string
  wa_error_message?: string
  chatType?: string
  assignee?: {
    full_name?: string
  }
  object?: any
  reactions?: any
  buttonLabel?: string|null
  isFromApplication: boolean | null
  applicationAgentName: string | null
}

let props = withDefaults(defineProps<Props>(), {
  isSeller: false,
  isError: false,
  isShowReaction: true,
  isShowMenuChat: true,
  is_forwarded: 0,
  isLoader: false,
  isLoading: false,
  buttonLabel: null,
  date: null,
  id: null,
  status: null,
  assignee: null,
  chatType: null,
  message: null,
  wa_error_message: null,
  reply_from_id: null,
  reactions: null,
  object: {
    replied_message: {
      message: null,
      name: null,
      type: null,
      object: null,
    }
  },
  isFromApplication: null,
  applicationAgentName: null,
});

const chat = useChatListStore()
const general = useGeneralStore()
const showEmoji = ref(false)
const emojiActive = ref(false)
const reactionArr = ref([])

const handleReactionSuccess = (emojiString: any) => {
  let type_chat = general.chatTab
  let platform = ''

  if(general.chatTab === 'instagram'){
    type_chat = 'instagram'

    if(chat.selectedChatType === 'instagram'){
      platform = 'instagram'
    }
    
    if(chat.selectedChatType === 'facebook'){
      platform = 'page'
    }
  }

  chat.sendEmoji(chat.selectedChat, {
    to: chat?.detail?.customer?.phone,
    chat_id: props.id,
    emoji: emojiString,
    platform: platform
  },type_chat).then((resp) => {
    if (resp.code === 200) {
      const currentReaction = reactionArr.value.findIndex((rc) => rc.chat_id === props.id && rc.from_chat_user_id === chat.selectedChat)
      if (currentReaction == -1 && reactionArr.value.length == 0) {
        reactionArr.value.push({
          chat_id : props?.id,
          from_chat_user_id: chat.selectedChat,
          value: emojiString
        })
      } else {

        reactionArr.value.splice(currentReaction,1, {
          chat_id : props?.id,
          from_chat_user_id: chat.selectedChat,
          value: emojiString
        })
      }
    } else {
      console.log("400")
    }
  })
}

const viewReply= () => {
  general.isShowReplyChat = !general.isShowReplyChat

  if(general.isShowReplyChat == true){
    general.isShowProfile = false
    general.replyChatId = props?.id
  }else{
    general.replyChatId = null
  }
}

const handleOnActive= (event) => {
  emojiActive.value = event
}

onMounted(() => {
  reactionArr.value = props.reactions
})

watch(() => props.reactions, (val) => {
  reactionArr.value = val
})


</script>
